/* This file is part of the ITRANS song book package.
 * avinash@acm.org
 * Converts punctuation between #indian and #endindian, surrounds it with ##
 * For use with song files or other ITRANS docs meant for display with
 * xdvng font (it does not contain punctuation characters).
 */
/*------------------------------------------------------------------*/
/* MS-DOS info: compile this into a .com file - tiny model.
 * bcc -N -mt -lt isongcat.c
 */
/* UNIX info: (not tested on UNIX), but a simple
 * cc -o isongcat isongcat.c
 * should work.
 */

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>

#define LEN_STRING 1024
#define FALSE 0
#define TRUE 1

/* MS-DOS */
#if defined(__MSDOS__) || defined(MSDOS)
#define FOPEN_READ_TEXT "rt"
/* don't use this, printing out songfile causes problems, and anyway,
 * / works just fine under dos too.
 *  #define DIR_SEP '\\' */
#else
/* UNIX */
#define FOPEN_READ_TEXT "r"
#endif

#define INDIAN_STR	"#indian"
#define EINDIAN_STR	"#endindian"
#define THREEDOTS_STR	"\\threedots"
#define STARTS_STR	"\\startsong"
#define ENDS_STR	"\\endsong"
#define PRTL_STR	"\\printtitle"

int main(int argc, char* argv[])
{
    char str[LEN_STRING];
    int i = 0, len = 0, lineno = 0, cmplen = 0;
    int change = FALSE;

    /* read file from stdin */
    while (fgets(str, sizeof(str), stdin)) {
	lineno++;
        len = strlen(str);
	//fprintf(stdout, "line %d change %d \n", lineno, change);
	for (i = 0; i < len; i ++) {
	    switch(str[i]) {
	    case '%':
	        /* skip all comments */
		if (i != 0) putchar('\n');
		    /* if entire line is a comment, delete it completely */
		i = len;
		break;
	    case '#':
	        if (strncmp(str+i, INDIAN_STR, strlen(INDIAN_STR)) == 0) {
		    if (str[i+strlen(INDIAN_STR)] == '='
		       || isalpha(str[i+strlen(INDIAN_STR)]))
			change = FALSE;
		    else
			change = TRUE;
		} else if (str[i+1] == '#') {
		    /* seen ## toggle #indian/#endindian */
		    change = !change;
		    putchar(str[i]);
		    i+=1;
		} else if (strncmp(str+i, EINDIAN_STR, strlen(EINDIAN_STR)) == 0) {
		    change = FALSE;
		}
		putchar(str[i]);
		break;
	    case ',':
	    case ';':
	    case '!':
	    case '/':
	    case '$':
	    case '*':
	    case '(':
	    case ')':
	    case '[':
	    case ']':
	    case '}':
	    case '{':
	    case '+':
	    case '-':
	    case '?':
	    case ':':
	    case '"':
	    case '\'':
	    case '`':
	    case '~':
	        if (change) {
		    printf("##%c##", str[i]);
		} else putchar(str[i]);
		break;
	    case '&':
	        if (change) {
		    printf("##&amp;##");
		} else putchar(str[i]);
		break;
	    case '.':
	    case '^':
	        if (change && !isalpha(str[i+1])) {
			/* leave .n .C ^n etc alone */
		    printf("##%c##", str[i]);
		} else putchar(str[i]);
		break;
	    case '\\':
		/* remove \commmands even when no change is set */
		if (strncmp(str+i, THREEDOTS_STR, strlen(THREEDOTS_STR)) == 0) {
		    printf("##. . .##");
		    i += (strlen(THREEDOTS_STR) - 1);
		} else if (strncmp(str+i, STARTS_STR, strlen(STARTS_STR)) == 0) {
		    /* remove \startsong lines, no printf */
		    i += (strlen(STARTS_STR) - 1);
		    if (str[i+1] == '\n') i++; /* if end of line, remove it */
		} else if (strncmp(str+i, ENDS_STR, strlen(ENDS_STR)) == 0) {
		    /* remove \endsong lines, no printf */
		    i += (strlen(ENDS_STR) - 1);
		    if (str[i+1] == '\n') i++; /* if end of line, remove it */
		} else if (strncmp(str+i, PRTL_STR, strlen(PRTL_STR)) == 0) {
		    /* remove \prittitle lines, no printf */
		    i += (strlen(PRTL_STR) - 1);
		    if (str[i+1] == '\n') i++; /* if end of line, remove it */
		} else if (change) {
		    /* eat up this char, and handle the next one normally
		       \- \& etc */
		} else putchar(str[i]);
		break;
	    default:
		putchar(str[i]);
		break;
	    } /* switch */
	} /* for */
    }
    return 0;
}
