/* This file is part of the ITRANS song book package.
 * avinash@acm.org
 * Converts grep *.s output, lines like:
1.s:\stitle{kevhaa tarii pahaaTe}%
 * into html lines, such as:
<br>
<a href="a/1.s" target="song">itr</a> :
<a href="a/xht/1.htm" target="song">kevhaa tarii pahaaTe </a>
 *
 * To use (creating a sorted index, sort by song title):
 grep -H stitle [a-z]/*s > all
 cat all | sort -f -t: +1 | st2html > out
 * Then cut lines from out into the hindex?.html files...
 ...

 * st2html can also handle lines like:
a/ghalib73.s:\stitle{bahot sahii Gam-e-getii sharaab kam kyaa hai}
1/2.s:\stitle{bahot sahii Gam-e-getii sharaab kam kyaa hai}
 */
/*------------------------------------------------------------------*/
/* MS-DOS info: compile this into a .com file - tiny model.
 * bcc -N -mt -lt st2html.c
 */
/* UNIX info: 
 * cc -o st2html st2html.c
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define LEN_STRING 1024
#define FALSE 0
#define TRUE 1

/* MS-DOS */
#if defined(__MSDOS__) || defined(MSDOS)
#define FOPEN_READ_TEXT "rt"
/* don't use this, printing out songfile causes problems, and anyway,
 * / works just fine under dos too.
 *  #define DIR_SEP '\\' */
#else
/* UNIX */
#define FOPEN_READ_TEXT "r"
#endif

#define INDIAN_STR	"#indian"
#define EINDIAN_STR	"#endindian"

#define ST_HTML_1 "<br><a href=\""
#define ST_HTML_2 "\" target=\"song\">%4d.s</a> :\n  <a href=\""
#define ST_HTML_2s "\" target=\"song\">%s.s</a> :\n  <a href=\""
#define ST_HTML_3 "\" target=\"song\">"
#define ST_HTML_4 "</a>"

#define HTML_DIR	"xht"

#define NUM_FILES	(500)

int main(int argc, char* argv[])
{
    char str[LEN_STRING];
    char stitle[LEN_STRING];
    char dirnames[26], dir;
    char *cptr1, *cptr2;
    int i = 0, len = 0, lineno = 0, fno = 0, dirno;
    int change = FALSE;

    /* files 1..500 are in folder a, 501..1000 in folder b, etc */
    for (i = 0; i < sizeof(dirnames)/sizeof(dirnames[0]); i ++) {
        dirnames[i] = 'a' + i;
    }

    /* read file from stdin */
    while (fgets(str, sizeof(str), stdin)) {
	lineno++;
        len = strlen(str);
	//fprintf(stdout, "line %d change %d \n", lineno, change);
	fno = atoi(str); /* 1.s returns 1, etc */
	cptr1 = strchr(str, '{');
	cptr2 = strrchr(str, '}');
	if (cptr1 && cptr2 && (cptr1 < cptr2)) { 
	    cptr1++; /* skip { */
	    *cptr2 = '\0'; /* } is end of string */
	    if (str[1] == '/') { /* 1/1.s or a/2.s or a/fname.s etc */
		dir = str[0];
		cptr2 = strchr(str, '.');
		*cptr2 = '\0';
		printf(ST_HTML_1);
		printf("%s.s", str);
		printf(ST_HTML_2s, &str[2]);
		printf("%c/%s/%s.htm", dir, HTML_DIR, &str[2]);
		printf(ST_HTML_3);
		printf(cptr1);
		printf(ST_HTML_4);
		putchar('\n');
	    } else if (fno > 0) {
		/* find directory name - a or b ... */
		dirno = (fno - 1) / NUM_FILES;
		dir = dirnames[dirno];

		printf(ST_HTML_1);
		printf("%c/%d.s", dir, fno);
		printf(ST_HTML_2, fno);
		printf("%c/%s/%d.htm", dir, HTML_DIR, fno);
		printf(ST_HTML_3);
		printf(cptr1);
		printf(ST_HTML_4);
		putchar('\n');
	    }
	} else {
	    fprintf(stderr, "Warning: line %d (%s) - found nothing\n",
	    lineno, str);
	}
    }
    return 0;
}
